@@ -185,7 +185,11 @@ get_xkb_desc_rec (MsdA11yKeyboardManager *manager)
                 desc->ctrls = NULL;
                 status = XkbGetControls (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), XkbAllControlsMask, desc);
         }
+#if GTK_CHECK_VERSION (3, 0, 0)
+        gdk_error_trap_pop_ignored ();
+#else
         gdk_error_trap_pop ();
+#endif
 
         g_return_val_if_fail (desc != NULL, NULL);
         g_return_val_if_fail (desc->ctrls != NULL, NULL);
@@ -392,7 +396,11 @@ set_server_from_settings (MsdA11yKeyboardManager *manager)
         XkbFreeKeyboard (desc, XkbAllComponentsMask, True);
 
         XSync (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), FALSE);
+#if GTK_CHECK_VERSION (3, 0, 0)
+        gdk_error_trap_pop_ignored ();
+#else
         gdk_error_trap_pop ();
+#endif
 
         mate_settings_profile_end (NULL);
 }
@@ -436,7 +444,7 @@ ax_response_callback (MsdA11yKeyboardManager *manager,
 
                 err = NULL;
                 if (!gtk_show_uri (screen,
-                                   "ghelp:user-guide#goscustaccess-6",
+                                   "help:user-guide#goscustaccess-6",
                                    gtk_get_current_event_time(),
                                    &err)) {
                         GtkWidget *error_dialog = gtk_message_dialog_new (parent,
@@ -1073,7 +1081,11 @@ restore_server_xkb_config (MsdA11yKeyboardManager *manager)
                          XkbAllComponentsMask, True);
 
         XSync (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), FALSE);
+#if GTK_CHECK_VERSION (3, 0, 0)
+        gdk_error_trap_pop_ignored ();
+#else
         gdk_error_trap_pop ();
+#endif
 
         manager->priv->original_xkb_desc = NULL;
 }
@@ -1115,76 +1127,11 @@ msd_a11y_keyboard_manager_stop (MsdA11yKeyboardManager *manager)
         p->stickykeys_shortcut_val = FALSE;
 }
 
-static void
-msd_a11y_keyboard_manager_set_property (GObject        *object,
-                                        guint           prop_id,
-                                        const GValue   *value,
-                                        GParamSpec     *pspec)
-{
-        MsdA11yKeyboardManager *self;
-
-        self = MSD_A11Y_KEYBOARD_MANAGER (object);
-
-        switch (prop_id) {
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
-}
-
-static void
-msd_a11y_keyboard_manager_get_property (GObject        *object,
-                                        guint           prop_id,
-                                        GValue         *value,
-                                        GParamSpec     *pspec)
-{
-        MsdA11yKeyboardManager *self;
-
-        self = MSD_A11Y_KEYBOARD_MANAGER (object);
-
-        switch (prop_id) {
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
-}
-
-static GObject *
-msd_a11y_keyboard_manager_constructor (GType                  type,
-                                       guint                  n_construct_properties,
-                                       GObjectConstructParam *construct_properties)
-{
-        MsdA11yKeyboardManager      *a11y_keyboard_manager;
-        MsdA11yKeyboardManagerClass *klass;
-
-        klass = MSD_A11Y_KEYBOARD_MANAGER_CLASS (g_type_class_peek (MSD_TYPE_A11Y_KEYBOARD_MANAGER));
-
-        a11y_keyboard_manager = MSD_A11Y_KEYBOARD_MANAGER (G_OBJECT_CLASS (msd_a11y_keyboard_manager_parent_class)->constructor (type,
-                                                                                                      n_construct_properties,
-                                                                                                      construct_properties));
-
-        return G_OBJECT (a11y_keyboard_manager);
-}
-
-static void
-msd_a11y_keyboard_manager_dispose (GObject *object)
-{
-        MsdA11yKeyboardManager *a11y_keyboard_manager;
-
-        a11y_keyboard_manager = MSD_A11Y_KEYBOARD_MANAGER (object);
-
-        G_OBJECT_CLASS (msd_a11y_keyboard_manager_parent_class)->dispose (object);
-}
-
 static void
 msd_a11y_keyboard_manager_class_init (MsdA11yKeyboardManagerClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-        object_class->get_property = msd_a11y_keyboard_manager_get_property;
-        object_class->set_property = msd_a11y_keyboard_manager_set_property;
-        object_class->constructor = msd_a11y_keyboard_manager_constructor;
-        object_class->dispose = msd_a11y_keyboard_manager_dispose;
         object_class->finalize = msd_a11y_keyboard_manager_finalize;
 
         g_type_class_add_private (klass, sizeof (MsdA11yKeyboardManagerPrivate));
